# firebase-hackernews [![Build Status](https://travis-ci.org/ragingwind/firebase-hackernews.svg?branch=master)](https://travis-ci.org/ragingwind/firebase-hackernews)

> Hacker News APIs with firebase

## Install

```
$ npm install --save firebase-hackernews
```
## Test

### Express

Run node script at `./example/express/server.js`, or npm run express and connect to server

### react

This project was generated by `create-react-app`. Navigate to `./example/react/` and then run first `npm install`, an then run `npm start`. If you have any updates before run `start`, you should reinstall the package

### Service Worker

To test service worker, run `npm run sw` and connet to test server. webpack watching is default options

## Usage

See [more examples](./examples) in test.js and refer to [HackerNews API](https://github.com/HackerNews/API) for more information of firebase and

### Initialzing with firebase/database

Because of firebase running environment is not only for node but also browser and service worker.So you must import both of app and database from `firebase` modules before import `firebase-hackernews`.

```js
const firebase = require('firebase');
const hackernews = require('firebase-hackernews');

// create a service as a single instance when the fist call
// you must pass firebase to init method
const hnservice = hackernews(firebase)
```

Even it can be running with es2015 to support for live-code importing like this below,

```js
import firebase from 'firebase/app'
import from 'firebase/database'

const hnservice = hackernews(firebase)
```

### With Promise

```js
// get all of stories by types, 'top', 'new', 'best', 'ask', 'show', 'job'
hnservice.stories('top').then(stories => {})

// get stories with custom count and page
hnservice.stories('top', {page: 1, count: 30}).then(stories => {})

// get a user
hnservice.user('jl').then(user => {})

// get a current max item id
hnservice.maxItem().then(update => {})

// get a updated items and profiles
hnservice.update().then(update => {})
```

### Without Promise

APIs named to xxxxCached is that support return data immediately but synchronous APIs doesn't do fetch. It only works on cached data that means asynchronous apis alreay has been called or running on watch mode

```js
hnservice.storiesCached('top').then(stories => {})
```

### Fetching with Service Worker

This module supports that running on server-worker. After initialzing in service worker, you can get a data via fetch. To do this, firstly, you must import and initialize both of firebase and hackernews service

```js
/* global importScripts hackernews */
importScripts('https://www.gstatic.com/firebasejs/4.1.2/firebase-app.js')
importScripts('https://www.gstatic.com/firebasejs/4.1.2/firebase-database.js')
importScripts('https://unpkg.com/@^1.0.0firebase-hackernews')

hackernews.init(firebase, { watch: true })
```

and then you can request a stories via fetch and subpath. see more examples in `examples/serice-worker`

```js
const stories = await fetch('./hackernews/top')
const storiesRes = await stories.json()

storiesRes.data.forEach(s => {
	// manage a story
})
```

## API

### init

Returns hackernews service powered by firebase as a single instance

### APIs for Promise

#### stories(type, [options])

Returns stories with `totalLength` as an additional info after fetched and cached with options:

- force: true ? returns stories fetch first, else return cached data if it exist
- page: returns stories in page by count
- count: count in a page. default is 50

#### items(id[s], [options])

Returns items by id[s] after fetched and cached with options:

- force: true ? returns stories cache first, else return cached data after fetch

#### user(id)

Returns profile by id

#### updates()

Returns recent updates regardless fetched data

#### maxItem()

Returnes max item id of latest snapshot on firebase

#### watch()

Make the service keep listening on the changes of stories. It recommend to use it for desktop application and server side. refer to [the example with express.js](./examples/express)

#### length(type)

Returns a length of cached items of the target type

#### kids(id)

Return cached all of items related to the target id. key is item's id and data will be flatted object list

#### data([data])

Set and get data, on cache directly. It's useful when it comes to hydrate / serialis cache

### APIs for cached data

Cached APIs returns with data immediately from chached data without fetch and Promise. It would be possible that data is not ready befor you do calll `cached` APIs

- storiesCached, identical to stories()
- itemsCached, identical to items()
- lengthCached, identical to length() but no promise
- dataCached, identical to data() but no promise

## License

MIT © [Jimmy Moon](http://ragingwind.me)
